@@ -26,7 +26,16 @@ enum calico_reason {
 	CALI_REASON_SOURCE_COLLISION,
 	CALI_REASON_SOURCE_COLLISION_FAILED,
 	CALI_REASON_CT_CREATE_FAILED,
-	CALI_REASON_ACCEPTED_BY_XDP, // Not used by counters map
+	CALI_REDIRECT,
+	CALI_REDIRECT_NEIGH,
+	CALI_REDIRECT_PEER,
+	CALI_REASON_FRAG_WAIT,
+	CALI_REASON_FRAG_REORDER,
+	CALI_REASON_FRAG_UNSUPPORTED,
+	CALI_REASON_DROPPED_BY_QOS,
+	RESERVED1,
+	// Not used by counters map
+	CALI_REASON_ACCEPTED_BY_XDP,
 	CALI_REASON_WEP_NOT_READY,
 	CALI_REASON_NATIFACE,
 };
