@@ -98,7 +98,8 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 	Expect(err).NotTo(HaveOccurred())
 
 	commonMaps := bpfmaps.CommonMaps
-	programs := commonMaps.ProgramsMap.(*hook.ProgramsMap)
+	programsIng := commonMaps.ProgramsMaps[hook.Ingress].(*hook.ProgramsMap)
+	programsEg := commonMaps.ProgramsMaps[hook.Egress].(*hook.ProgramsMap)
 	loglevel := "off"
 
 	bpfEpMgr, err := newBPFTestEpMgr(
@@ -115,6 +116,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			BPFExtToServiceConnmark: 0,
 			BPFPolicyDebugEnabled:   true,
 			BPFIpv6Enabled:          ipv6Enabled,
+			BPFAttachType:           "TCX",
 		},
 		bpfmaps,
 		regexp.MustCompile("^workloadep[0123]"),
@@ -139,13 +141,17 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programsCount := 14
+		programsIngCount := 7
+		programsEgCount := 7
 		if ipv6Enabled {
-			programsCount = 27
+			programsIngCount = 14
+			programsEgCount = 13
 		}
-		Expect(programs.Count()).To(Equal(programsCount))
-		at := programs.Programs()
-		Expect(at).To(HaveKey(hook.AttachType{
+		Expect(programsIng.Count()).To(Equal(programsIngCount))
+		Expect(programsEg.Count()).To(Equal(programsEgCount))
+		atIng := programsIng.Programs()
+		atEg := programsEg.Programs()
+		Expect(atIng).To(HaveKey(hook.AttachType{
 			Hook:       hook.Ingress,
 			Family:     4,
 			Type:       tcdefs.EpTypeHost,
@@ -153,7 +159,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			FIB:        true,
 			ToHostDrop: false,
 			DSR:        false}))
-		Expect(at).To(HaveKey(hook.AttachType{
+		Expect(atEg).To(HaveKey(hook.AttachType{
 			Hook:       hook.Egress,
 			Family:     4,
 			Type:       tcdefs.EpTypeHost,
@@ -161,7 +167,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			FIB:        true,
 			ToHostDrop: false,
 			DSR:        false}))
-		Expect(at).NotTo(HaveKey(hook.AttachType{
+		Expect(atIng).NotTo(HaveKey(hook.AttachType{
 			Hook:       hook.Ingress,
 			Family:     6,
 			Type:       tcdefs.EpTypeHost,
@@ -169,7 +175,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			FIB:        true,
 			ToHostDrop: false,
 			DSR:        false}))
-		Expect(at).NotTo(HaveKey(hook.AttachType{
+		Expect(atEg).NotTo(HaveKey(hook.AttachType{
 			Hook:       hook.Egress,
 			Family:     6,
 			Type:       tcdefs.EpTypeHost,
@@ -195,11 +201,12 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			bpfEpMgr.OnUpdate(&proto.HostMetadataV6Update{Hostname: "uthost", Ipv6Addr: "1::4"})
 			err = bpfEpMgr.CompleteDeferredWork()
 			Expect(err).NotTo(HaveOccurred())
-			Expect(programs.Count()).To(Equal(52))
+			Expect(programsIng.Count()).To(Equal(26))
 
-			at := programs.Programs()
+			atIng := programsIng.Programs()
+			atEg := programsEg.Programs()
 
-			Expect(at).To(HaveKey(hook.AttachType{
+			Expect(atIng).To(HaveKey(hook.AttachType{
 				Hook:       hook.Ingress,
 				Family:     4,
 				Type:       tcdefs.EpTypeHost,
@@ -207,7 +214,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 				FIB:        true,
 				ToHostDrop: false,
 				DSR:        false}))
-			Expect(at).To(HaveKey(hook.AttachType{
+			Expect(atEg).To(HaveKey(hook.AttachType{
 				Hook:       hook.Egress,
 				Family:     4,
 				Type:       tcdefs.EpTypeHost,
@@ -215,7 +222,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 				FIB:        true,
 				ToHostDrop: false,
 				DSR:        false}))
-			Expect(at).To(HaveKey(hook.AttachType{
+			Expect(atIng).To(HaveKey(hook.AttachType{
 				Hook:       hook.Ingress,
 				Family:     6,
 				Type:       tcdefs.EpTypeHost,
@@ -223,7 +230,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 				FIB:        true,
 				ToHostDrop: false,
 				DSR:        false}))
-			Expect(at).To(HaveKey(hook.AttachType{
+			Expect(atEg).To(HaveKey(hook.AttachType{
 				Hook:       hook.Egress,
 				Family:     6,
 				Type:       tcdefs.EpTypeHost,
@@ -272,13 +279,14 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 
 		}
 
-		pm := jumpMapDump(commonMaps.JumpMap)
-		Expect(pm).To(HaveKey(hostep1State.IngressPolicyV4()))
-		Expect(pm).To(HaveKey(hostep1State.EgressPolicyV4()))
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+		Expect(pmIng).To(HaveKey(hostep1State.IngressPolicyV4()))
+		Expect(pmEgr).To(HaveKey(hostep1State.EgressPolicyV4()))
 
 		if ipv6Enabled {
-			Expect(pm).To(HaveKey(hostep1State.IngressPolicyV6()))
-			Expect(pm).To(HaveKey(hostep1State.EgressPolicyV6()))
+			Expect(pmIng).To(HaveKey(hostep1State.IngressPolicyV6()))
+			Expect(pmEgr).To(HaveKey(hostep1State.EgressPolicyV6()))
 
 		}
 
@@ -335,35 +343,46 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err := bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programCount := 14
-		jumpMapLen := 2
+		programIngCount := 7
+		programEgCount := 7
+		jumpMapLen := 1
 		if ipv6Enabled {
-			programCount = 52
-			jumpMapLen = 8
+			programIngCount = 26
+			programEgCount = 26
+			jumpMapLen = 4
 		}
-		Expect(programs.Count()).To(Equal(programCount))
+		Expect(programsIng.Count()).To(Equal(programIngCount))
+		Expect(programsEg.Count()).To(Equal(programEgCount))
 
-		pm := jumpMapDump(commonMaps.JumpMap)
-		Expect(len(pm)).To(Equal(jumpMapLen)) // no policy for hep2
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+		Expect(len(pmIng)).To(Equal(jumpMapLen)) // no policy for hep2
+		Expect(len(pmEgr)).To(Equal(jumpMapLen)) // no policy for hep2
 	})
 
 	workload1 := createVethName("workloadep1")
 	defer deleteLink(workload1)
 
 	t.Run("create a workload", func(t *testing.T) {
+		programsIngCount := programsIng.Count()
+		programsEgCount := programsEg.Count()
 		bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep1", ifacemonitor.StateUp, workload1.Attrs().Index))
 		bpfEpMgr.OnUpdate(linux.NewIfaceAddrsUpdate("workloadep1", "1.6.6.6"))
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programsCount := 27
+		programsIngCount = programsIngCount + 7
+		programsEgCount = programsEgCount + 6
 		if ipv6Enabled {
-			programsCount = 52
+			programsIngCount = 26
+			programsEgCount = 26
 		}
-		Expect(programs.Count()).To(Equal(programsCount))
+		Expect(programsIng.Count()).To(Equal(programsIngCount))
+		Expect(programsEg.Count()).To(Equal(programsEgCount))
 
-		at := programs.Programs()
-		Expect(at).To(HaveKey(hook.AttachType{
+		atIng := programsIng.Programs()
+		atEg := programsEg.Programs()
+		Expect(atIng).To(HaveKey(hook.AttachType{
 			Hook:       hook.Ingress,
 			Family:     4,
 			Type:       tcdefs.EpTypeWorkload,
@@ -371,7 +390,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			FIB:        true,
 			ToHostDrop: false,
 			DSR:        false}))
-		Expect(at).To(HaveKey(hook.AttachType{
+		Expect(atEg).To(HaveKey(hook.AttachType{
 			Hook:       hook.Egress,
 			Family:     4,
 			Type:       tcdefs.EpTypeWorkload,
@@ -380,7 +399,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			ToHostDrop: false,
 			DSR:        false}))
 		if ipv6Enabled {
-			Expect(at).To(HaveKey(hook.AttachType{
+			Expect(atIng).To(HaveKey(hook.AttachType{
 				Hook:       hook.Ingress,
 				Family:     6,
 				Type:       tcdefs.EpTypeWorkload,
@@ -388,7 +407,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 				FIB:        true,
 				ToHostDrop: false,
 				DSR:        false}))
-			Expect(at).To(HaveKey(hook.AttachType{
+			Expect(atEg).To(HaveKey(hook.AttachType{
 				Hook:       hook.Egress,
 				Family:     6,
 				Type:       tcdefs.EpTypeWorkload,
@@ -404,16 +423,17 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		Expect(wl1State.EgressPolicyV4()).NotTo(Equal(-1))
 		Expect(wl1State.XDPPolicyV4()).To(Equal(-1))
 
-		pm := jumpMapDump(commonMaps.JumpMap)
-		Expect(pm).To(HaveKey(wl1State.IngressPolicyV4()))
-		Expect(pm).To(HaveKey(wl1State.EgressPolicyV4()))
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+		Expect(pmIng).To(HaveKey(wl1State.IngressPolicyV4()))
+		Expect(pmEgr).To(HaveKey(wl1State.EgressPolicyV4()))
 		if ipv6Enabled {
 			Expect(wl1State.IngressPolicyV6()).NotTo(Equal(-1))
 			Expect(wl1State.EgressPolicyV6()).NotTo(Equal(-1))
 			Expect(wl1State.XDPPolicyV6()).To(Equal(-1))
 
-			Expect(pm).To(HaveKey(wl1State.IngressPolicyV6()))
-			Expect(pm).To(HaveKey(wl1State.EgressPolicyV6()))
+			Expect(pmIng).To(HaveKey(wl1State.IngressPolicyV6()))
+			Expect(pmEgr).To(HaveKey(wl1State.EgressPolicyV6()))
 		}
 	})
 
@@ -421,22 +441,25 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 	defer deleteLink(workload2)
 
 	t.Run("create another workload, should not load more than the preable", func(t *testing.T) {
+		programsIngCount := programsIng.Count()
+		programsEgCount := programsEg.Count()
 		bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep2", ifacemonitor.StateUp, workload2.Attrs().Index))
 		bpfEpMgr.OnUpdate(linux.NewIfaceAddrsUpdate("workloadep2", "1.6.6.1"))
 		err := bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programsCount := 27
-		jumpMapLen := 6
+		jumpMapLen := 3
 
 		if ipv6Enabled {
-			programsCount = 52
-			jumpMapLen = 16
+			jumpMapLen = 8
 		}
-		Expect(programs.Count()).To(Equal(programsCount))
+		Expect(programsIng.Count()).To(Equal(programsIngCount))
+		Expect(programsEg.Count()).To(Equal(programsEgCount))
 
-		pm := jumpMapDump(commonMaps.JumpMap)
-		Expect(len(pm)).To(Equal((jumpMapLen)))
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+		Expect(len(pmIng)).To(Equal((jumpMapLen)))
+		Expect(len(pmEgr)).To(Equal((jumpMapLen)))
 	})
 
 	t.Run("bring first host ep down, should clean up its policies", func(t *testing.T) {
@@ -445,14 +468,15 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err := bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		pm := jumpMapDump(commonMaps.JumpMap)
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
 		// We remember the state from above
-		Expect(pm).NotTo(HaveKey(hostep1State.IngressPolicyV4()))
-		Expect(pm).NotTo(HaveKey(hostep1State.EgressPolicyV4()))
+		Expect(pmIng).NotTo(HaveKey(hostep1State.IngressPolicyV4()))
+		Expect(pmEgr).NotTo(HaveKey(hostep1State.EgressPolicyV4()))
 
 		if ipv6Enabled {
-			Expect(pm).NotTo(HaveKey(hostep1State.IngressPolicyV6()))
-			Expect(pm).NotTo(HaveKey(hostep1State.EgressPolicyV6()))
+			Expect(pmIng).NotTo(HaveKey(hostep1State.IngressPolicyV6()))
+			Expect(pmEgr).NotTo(HaveKey(hostep1State.EgressPolicyV6()))
 		}
 		xdppm := jumpMapDump(commonMaps.XDPJumpMap)
 		Expect(xdppm).To(HaveLen(0))
@@ -465,9 +489,10 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		wl1State = ifstateMap[ifstate.NewKey(uint32(workload1.Attrs().Index))]
 		fmt.Printf("wl1State = %+v\n", wl1State)
 
-		pm := jumpMapDump(commonMaps.JumpMap)
-		wl1IngressPol := pm[wl1State.IngressPolicyV4()]
-		wl1EgressPol := pm[wl1State.EgressPolicyV4()]
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+		wl1IngressPol := pmIng[wl1State.IngressPolicyV4()]
+		wl1EgressPol := pmEgr[wl1State.EgressPolicyV4()]
 
 		bpfEpMgr.OnUpdate(&proto.WorkloadEndpointUpdate{
 			Id: &proto.WorkloadEndpointID{
@@ -497,9 +522,10 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		Expect(wl1State2).To(Equal(wl1State))
 
 		// ... but the policy programs changed
-		pm = jumpMapDump(commonMaps.JumpMap)
-		Expect(wl1IngressPol).NotTo(Equal(pm[wl1State2.IngressPolicyV4()]))
-		Expect(wl1EgressPol).NotTo(Equal(pm[wl1State2.IngressPolicyV4()]))
+		pmIng = jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr = jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+		Expect(wl1IngressPol).NotTo(Equal(pmIng[wl1State2.IngressPolicyV4()]))
+		Expect(wl1EgressPol).NotTo(Equal(pmEgr[wl1State2.EgressPolicyV4()]))
 
 		progs, err := bpf.GetAllProgs()
 		Expect(err).NotTo(HaveOccurred())
@@ -514,10 +540,11 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err := bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		pm := jumpMapDump(commonMaps.JumpMap)
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
 		// We remember the state from above
-		Expect(pm).NotTo(HaveKey(wl1State.IngressPolicyV4()))
-		Expect(pm).NotTo(HaveKey(wl1State.EgressPolicyV4()))
+		Expect(pmIng).NotTo(HaveKey(wl1State.IngressPolicyV4()))
+		Expect(pmEgr).NotTo(HaveKey(wl1State.EgressPolicyV4()))
 	})
 
 	t.Run("restart", func(t *testing.T) {
@@ -547,7 +574,8 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		Expect(attached).To(HaveKey("hostep2"))
 		Expect(attached).NotTo(HaveKey("workloadep3"))
 
-		programs.ResetForTesting() // Because we recycle it, restarted Felix would get a fresh copy.
+		programsIng.ResetForTesting() // Because we recycle it, restarted Felix would get a fresh copy.
+		programsEg.ResetForTesting()  // Because we recycle it, restarted Felix would get a fresh copy.
 
 		bpfEpMgr, err = newBPFTestEpMgr(
 			&linux.Config{
@@ -585,21 +613,21 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 
 		// And they still have the same data so that existing preamble programs
 		// and policies can still point to the right stuff.
-		oldProgsParams := hook.ProgramsMapParameters
-		oldProgsParams.PinDir = tmp
-		oldProgs := maps.NewPinnedMap(oldProgsParams)
+		oldProgsIngParams := hook.IngressProgramsMapParameters
+		oldProgsIngParams.PinDir = tmp
+		oldProgs := maps.NewPinnedMap(oldProgsIngParams)
 		err = oldProgs.Open()
 		Expect(err).NotTo(HaveOccurred())
 		pm := jumpMapDump(oldProgs)
-		programsCount := 27
-		oldPoliciesCount := 4
+		programsCount := 14
+		oldPoliciesCount := 2
 		if ipv6Enabled {
-			programsCount = 52
-			oldPoliciesCount = 12
+			programsCount = 26
+			oldPoliciesCount = 6
 		}
 		Expect(pm).To(HaveLen(programsCount))
 
-		oldPoliciesParams := jump.MapParameters
+		oldPoliciesParams := jump.IngressMapParameters
 		oldPoliciesParams.PinDir = tmp
 		oldPolicies := maps.NewPinnedMap(oldPoliciesParams)
 		err = oldPolicies.Open()
@@ -608,10 +636,14 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		Expect(pm).To(HaveLen(oldPoliciesCount))
 
 		// After restat we get new maps which are empty
-		Expect(programs.Count()).To(Equal(0))
-		pm = jumpMapDump(commonMaps.ProgramsMap)
+		Expect(programsIng.Count()).To(Equal(0))
+		pm = jumpMapDump(commonMaps.ProgramsMaps[hook.Ingress])
 		Expect(pm).To(HaveLen(0))
-		pm = jumpMapDump(commonMaps.JumpMap)
+		pm = jumpMapDump(commonMaps.ProgramsMaps[hook.Egress])
+		Expect(pm).To(HaveLen(0))
+		pm = jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		Expect(pm).To(HaveLen(0))
+		pm = jumpMapDump(commonMaps.JumpMaps[hook.Egress])
 		Expect(pm).To(HaveLen(0))
 
 		err = bpfEpMgr.CompleteDeferredWork()
@@ -630,15 +662,17 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		Expect(programs.Count()).To(Equal(27))
-		pm = jumpMapDump(commonMaps.ProgramsMap)
-		Expect(pm).To(HaveLen(27))
+		Expect(programsIng.Count()).To(Equal(14))
+		pm = jumpMapDump(commonMaps.ProgramsMaps[hook.Ingress])
+		Expect(pm).To(HaveLen(14))
 
-		pm = jumpMapDump(commonMaps.JumpMap)
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
 		// We remember the state from above
-		Expect(pm).To(HaveLen(2))
-		Expect(pm).To(HaveKey(wl2State.IngressPolicyV4()))
-		Expect(pm).To(HaveKey(wl2State.EgressPolicyV4()))
+		Expect(pmIng).To(HaveLen(1))
+		Expect(pmEgr).To(HaveLen(1))
+		Expect(pmIng).To(HaveKey(wl2State.IngressPolicyV4()))
+		Expect(pmEgr).To(HaveKey(wl2State.EgressPolicyV4()))
 
 		_, err = os.Stat(path.Join(bpfdefs.GlobalPinDir, "old_jumps"))
 		Expect(err).To(HaveOccurred())
@@ -673,7 +707,8 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 
 		deleteLink(workload3)
 
-		programs.ResetForTesting() // Because we recycle it, restarted Felix would get a fresh copy.
+		programsIng.ResetForTesting() // Because we recycle it, restarted Felix would get a fresh copy.
+		programsEg.ResetForTesting()  // Because we recycle it, restarted Felix would get a fresh copy.
 
 		bpfEpMgr, err = newBPFTestEpMgr(
 			&linux.Config{
@@ -694,8 +729,11 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		)
 		Expect(err).NotTo(HaveOccurred())
 
-		pm := jumpMapDump(commonMaps.JumpMap)
-		Expect(pm).To(HaveLen(0))
+		pmIng := jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr := jumpMapDump(commonMaps.JumpMaps[hook.Egress])
+
+		Expect(pmIng).To(HaveLen(0))
+		Expect(pmEgr).To(HaveLen(0))
 
 		bpfEpMgr.OnUpdate(&proto.HostMetadataUpdate{Hostname: "uthost", Ipv4Addr: "1.2.3.4"})
 		bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep2", ifacemonitor.StateUp, workload2.Attrs().Index))
@@ -705,11 +743,13 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		pm = jumpMapDump(commonMaps.JumpMap)
+		pmIng = jumpMapDump(commonMaps.JumpMaps[hook.Ingress])
+		pmEgr = jumpMapDump(commonMaps.JumpMaps[hook.Egress])
 		// We remember the state from above
-		Expect(pm).To(HaveLen(2))
-		Expect(pm).To(HaveKey(wl2State.IngressPolicyV4()))
-		Expect(pm).To(HaveKey(wl2State.EgressPolicyV4()))
+		Expect(pmIng).To(HaveLen(1))
+		Expect(pmEgr).To(HaveLen(1))
+		Expect(pmIng).To(HaveKey(wl2State.IngressPolicyV4()))
+		Expect(pmEgr).To(HaveKey(wl2State.EgressPolicyV4()))
 	})
 }
 
@@ -729,7 +769,8 @@ func TestAttachWithMultipleWorkloadUpdate(t *testing.T) {
 	Expect(err).NotTo(HaveOccurred())
 
 	commonMaps := bpfmaps.CommonMaps
-	programs := commonMaps.ProgramsMap.(*hook.ProgramsMap)
+	programsIng := commonMaps.ProgramsMaps[hook.Ingress].(*hook.ProgramsMap)
+	programsEg := commonMaps.ProgramsMaps[hook.Egress].(*hook.ProgramsMap)
 	loglevel := "off"
 
 	bpfEpMgr, err := newBPFTestEpMgr(
@@ -802,8 +843,9 @@ func TestAttachWithMultipleWorkloadUpdate(t *testing.T) {
 	Expect(qosVal2.PacketRateTokens()).To(Equal(int16(-1)))
 	Expect(qosVal2.PacketRateLastUpdate()).To(Equal(uint64(0)))
 
-	at := programs.Programs()
-	Expect(at).To(HaveKey(hook.AttachType{
+	atIng := programsIng.Programs()
+	atEg := programsEg.Programs()
+	Expect(atIng).To(HaveKey(hook.AttachType{
 		Hook:       hook.Ingress,
 		Family:     4,
 		Type:       tcdefs.EpTypeWorkload,
@@ -811,7 +853,7 @@ func TestAttachWithMultipleWorkloadUpdate(t *testing.T) {
 		FIB:        true,
 		ToHostDrop: false,
 		DSR:        false}))
-	Expect(at).To(HaveKey(hook.AttachType{
+	Expect(atEg).To(HaveKey(hook.AttachType{
 		Hook:       hook.Egress,
 		Family:     4,
 		Type:       tcdefs.EpTypeWorkload,
@@ -969,7 +1011,7 @@ func TestCTLBAttachLegacy(t *testing.T) {
 		Expect(err).NotTo(HaveOccurred())
 
 		commonMaps := bpfmaps.CommonMaps
-		err = nat.InstallConnectTimeLoadBalancerLegacy(v4, v6, "", "debug", 60*time.Second, false, commonMaps.CTLBProgramsMap)
+		err = nat.InstallConnectTimeLoadBalancerLegacy(v4, v6, "", "debug", 60*time.Second, false, commonMaps.CTLBProgramsMaps)
 		Expect(err).NotTo(HaveOccurred())
 
 		checkPinPath := func(pinPath string, mustExist bool) {
@@ -1043,7 +1085,7 @@ func TestCTLBAttach(t *testing.T) {
 		Expect(err).NotTo(HaveOccurred())
 
 		commonMaps := bpfmaps.CommonMaps
-		err = nat.InstallConnectTimeLoadBalancer(v4, v6, "", "debug", 60*time.Second, false, commonMaps.CTLBProgramsMap)
+		err = nat.InstallConnectTimeLoadBalancer(v4, v6, "", "debug", 60*time.Second, false, commonMaps.CTLBProgramsMaps)
 		Expect(err).NotTo(HaveOccurred())
 
 		checkPinPath := func(pinPath string, mustExist bool) {
