@@ -25,7 +25,6 @@ import (
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	api "github.com/projectcalico/api/pkg/apis/projectcalico/v3"
-	v3 "github.com/projectcalico/api/pkg/apis/projectcalico/v3"
 	"github.com/projectcalico/api/pkg/lib/numorstring"
 
 	"github.com/projectcalico/calico/felix/fv/connectivity"
@@ -84,7 +83,7 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 					Skip("Skipping NFT / BPF tests for etcdv3 backend.")
 				}
 
-				topologyOptions = createBaseTopologyOptions(vxlanMode, enableIPv6, routeSource, brokenXSum)
+				topologyOptions = createVXLANBaseTopologyOptions(vxlanMode, enableIPv6, routeSource, brokenXSum)
 				topologyOptions.FelixLogSeverity = "Debug"
 
 				cc = &connectivity.Checker{}
@@ -99,7 +98,7 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				assignTunnelAddresses(infra, tc, client)
 			})
 
-			AfterEach(func() {
+			JustAfterEach(func() {
 				if CurrentGinkgoTestDescription().Failed {
 					for _, felix := range felixes {
 						if NFTMode() {
@@ -114,9 +113,14 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 						if enableIPv6 {
 							felix.Exec("ip", "-6", "route")
 						}
+						felix.Exec("ip", "-d", "link")
 					}
-				}
 
+					infra.DumpErrorData()
+				}
+			})
+
+			AfterEach(func() {
 				for _, wl := range w {
 					wl.Stop()
 				}
@@ -130,10 +134,6 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 					wl.Stop()
 				}
 				tc.Stop()
-
-				if CurrentGinkgoTestDescription().Failed {
-					infra.DumpErrorData()
-				}
 				infra.Stop()
 			})
 
@@ -719,7 +719,7 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				}
 			})
 
-			It("should configure the vxlan device correctly", func() {
+			_ = !BPFMode() && It("should configure the vxlan device correctly", func() {
 				// The VXLAN device should appear with default MTU, etc. FV environment uses MTU 1500,
 				// which means that we should expect 1450 after subtracting VXLAN overhead for IPv4 or 1430 for IPv6.
 				mtuStr := "mtu 1450"
@@ -842,12 +842,16 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				// Wait for the VXLAN device to be created.
 				mtuStr := "mtu 1450"
 				mtuStrV6 := "mtu 1430"
+				if BPFMode() {
+					mtuStr = "mtu 1500"
+					mtuStrV6 = "mtu 1500"
+				}
 				for _, felix := range felixes {
 					Eventually(func() string {
 						out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan.calico")
 						return out
 					}, "60s", "500ms").Should(ContainSubstring(mtuStr))
-					if enableIPv6 {
+					if !BPFMode() && enableIPv6 {
 						Eventually(func() string {
 							out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan-v6.calico")
 							return out
@@ -863,19 +867,22 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				_, err := client.FelixConfigurations().Create(context.Background(), felixConfig, options.SetOptions{})
 				Expect(err).NotTo(HaveOccurred())
 
-				// Expect the ipv4 VXLAN device to be deleted.
-				for _, felix := range felixes {
-					Eventually(func() string {
-						out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan.calico")
-						return out
-					}, "60s", "500ms").ShouldNot(ContainSubstring(mtuStr))
-					// IPv6 ignores the VXLAN enabled flag and must be disabled at the pool level. As such the ipv6
-					// interfaces should still exist at this point
-					if enableIPv6 {
+				if !BPFMode() || !enableIPv6 {
+					// Expect the ipv4 VXLAN device to be deleted. In BPFMode
+					// the same device is used for V6 as well
+					for _, felix := range felixes {
 						Eventually(func() string {
-							out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan-v6.calico")
+							out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan.calico")
 							return out
-						}, "60s", "500ms").Should(ContainSubstring(mtuStrV6))
+						}, "60s", "500ms").ShouldNot(ContainSubstring(mtuStr))
+						// IPv6 ignores the VXLAN enabled flag and must be disabled at the pool level. As such the ipv6
+						// interfaces should still exist at this point
+						if enableIPv6 {
+							Eventually(func() string {
+								out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan-v6.calico")
+								return out
+							}, "60s", "500ms").Should(ContainSubstring(mtuStrV6))
+						}
 					}
 				}
 
@@ -889,6 +896,10 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 					// Expect the ipv6 VXLAN device to be deleted.
 					for _, felix := range felixes {
 						Eventually(func() string {
+							if BPFMode() {
+								out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan.calico")
+								return out
+							}
 							out, _ := felix.ExecOutput("ip", "-d", "link", "show", "vxlan-v6.calico")
 							return out
 						}, "60s", "500ms").ShouldNot(ContainSubstring(mtuStrV6))
@@ -918,9 +929,9 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 					Skip("Skipping NFT / BPF tests for etcdv3 backend.")
 				}
 
-				topologyOptions = createBaseTopologyOptions(vxlanMode, enableIPv6, routeSource, brokenXSum)
+				topologyOptions = createVXLANBaseTopologyOptions(vxlanMode, enableIPv6, routeSource, brokenXSum)
 				topologyOptions.FelixLogSeverity = "Debug"
-				topologyOptions.VXLANStrategy = infrastructure.NewBorrowedIPVXLANStrategy(topologyOptions.IPPoolCIDR, topologyOptions.IPv6PoolCIDR, 3)
+				topologyOptions.VXLANStrategy = infrastructure.NewBorrowedIPTunnelStrategy(topologyOptions.IPPoolCIDR, topologyOptions.IPv6PoolCIDR, 3)
 
 				cc = &connectivity.Checker{}
 
@@ -946,8 +957,10 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 						felix.Exec("ipset", "list")
 						felix.Exec("ip", "r")
 						felix.Exec("ip", "a")
+						felix.Exec("calico-bpf", "routes", "dump")
 						if enableIPv6 {
 							felix.Exec("ip", "-6", "route")
+							felix.Exec("calico-bpf", "-6", "routes", "dump")
 						}
 					}
 				}
@@ -1015,12 +1028,12 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 					Skip("Skipping NFT / BPF tests for etcdv3 backend.")
 				}
 
-				topologyOptions = createBaseTopologyOptions(vxlanMode, enableIPv6, routeSource, brokenXSum)
+				topologyOptions = createVXLANBaseTopologyOptions(vxlanMode, enableIPv6, routeSource, brokenXSum)
 				topologyOptions.FelixLogSeverity = "Debug"
 
 				// Configure the default IP pool to be used for workloads only.
-				topologyOptions.IPPoolUsages = []api.IPPoolAllowedUse{v3.IPPoolAllowedUseWorkload}
-				topologyOptions.IPv6PoolUsages = []api.IPPoolAllowedUse{v3.IPPoolAllowedUseWorkload}
+				topologyOptions.IPPoolUsages = []api.IPPoolAllowedUse{api.IPPoolAllowedUseWorkload}
+				topologyOptions.IPv6PoolUsages = []api.IPPoolAllowedUse{api.IPPoolAllowedUseWorkload}
 
 				// Create a separate IP pool for tunnel addresses that uses /32 addresses.
 				tunnelPool := api.NewIPPool()
@@ -1028,7 +1041,7 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				tunnelPool.Spec.CIDR = "10.66.0.0/16"
 				tunnelPool.Spec.BlockSize = 32
 				tunnelPool.Spec.VXLANMode = vxlanMode
-				tunnelPool.Spec.AllowedUses = []api.IPPoolAllowedUse{v3.IPPoolAllowedUseTunnel}
+				tunnelPool.Spec.AllowedUses = []api.IPPoolAllowedUse{api.IPPoolAllowedUseTunnel}
 				cli := infra.GetCalicoClient()
 				_, err := cli.IPPools().Create(context.Background(), tunnelPool, options.SetOptions{})
 				Expect(err).NotTo(HaveOccurred())
@@ -1039,12 +1052,12 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				tunnelPoolV6.Spec.CIDR = "dead:feed::/64"
 				tunnelPoolV6.Spec.BlockSize = 128
 				tunnelPoolV6.Spec.VXLANMode = vxlanMode
-				tunnelPoolV6.Spec.AllowedUses = []api.IPPoolAllowedUse{v3.IPPoolAllowedUseTunnel}
+				tunnelPoolV6.Spec.AllowedUses = []api.IPPoolAllowedUse{api.IPPoolAllowedUseTunnel}
 				_, err = cli.IPPools().Create(context.Background(), tunnelPoolV6, options.SetOptions{})
 				Expect(err).NotTo(HaveOccurred())
 
 				// Configure the VXLAN strategy to use this IP pool for tunnel addresses allocation.
-				topologyOptions.VXLANStrategy = infrastructure.NewDefaultVXLANStrategy(tunnelPool.Spec.CIDR, tunnelPoolV6.Spec.CIDR)
+				topologyOptions.VXLANStrategy = infrastructure.NewDefaultTunnelStrategy(tunnelPool.Spec.CIDR, tunnelPoolV6.Spec.CIDR)
 
 				cc = &connectivity.Checker{}
 
@@ -1073,6 +1086,7 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 						if enableIPv6 {
 							felix.Exec("ip", "-6", "route")
 						}
+						felix.Exec("calico-bpf", "routes", "dump")
 					}
 				}
 
@@ -1127,15 +1141,14 @@ var _ = infrastructure.DatastoreDescribe("_BPF-SAFE_ VXLAN topology before addin
 				cc.CheckConnectivity()
 			})
 		})
-
 	}
 })
 
-func createBaseTopologyOptions(vxlanMode api.VXLANMode, enableIPv6 bool, routeSource string, brokenXSum bool) infrastructure.TopologyOptions {
+func createVXLANBaseTopologyOptions(vxlanMode api.VXLANMode, enableIPv6 bool, routeSource string, brokenXSum bool) infrastructure.TopologyOptions {
 	topologyOptions := infrastructure.DefaultTopologyOptions()
 	topologyOptions.VXLANMode = vxlanMode
-	topologyOptions.VXLANStrategy = infrastructure.NewDefaultVXLANStrategy(topologyOptions.IPPoolCIDR, topologyOptions.IPv6PoolCIDR)
-	topologyOptions.IPIPEnabled = false
+	topologyOptions.VXLANStrategy = infrastructure.NewDefaultTunnelStrategy(topologyOptions.IPPoolCIDR, topologyOptions.IPv6PoolCIDR)
+	topologyOptions.IPIPMode = api.IPIPModeNever
 	topologyOptions.EnableIPv6 = enableIPv6
 	topologyOptions.ExtraEnvVars["FELIX_ROUTESOURCE"] = routeSource
 	// We force the broken checksum handling on or off so that we're not dependent on kernel version
@@ -1143,6 +1156,7 @@ func createBaseTopologyOptions(vxlanMode api.VXLANMode, enableIPv6 bool, routeSo
 	// tested but we can verify the state with ethtool.
 	topologyOptions.ExtraEnvVars["FELIX_FeatureDetectOverride"] = fmt.Sprintf("ChecksumOffloadBroken=%t", brokenXSum)
 	topologyOptions.FelixDebugFilenameRegex = "vxlan|route_table|l3_route_resolver|int_dataplane"
+	topologyOptions.ExtraEnvVars["FELIX_BPFLogLevel"] = "off"
 	return topologyOptions
 }
 
@@ -1177,42 +1191,36 @@ func assignTunnelAddresses(infra infrastructure.DatastoreInfra, tc infrastructur
 			})
 			Expect(err).NotTo(HaveOccurred(), "failed to assign VXLAN v6 tunnel address")
 		}
+		if f.ExpectedIPIPTunnelAddr != "" {
+			handle := fmt.Sprintf("ipip-tunnel-addr-%s", f.Hostname)
+			err := client.IPAM().AssignIP(context.Background(), ipam.AssignIPArgs{
+				IP:       net.MustParseIP(f.ExpectedIPIPTunnelAddr),
+				HandleID: &handle,
+				Attrs: map[string]string{
+					ipam.AttributeNode: f.Hostname,
+					ipam.AttributeType: ipam.AttributeTypeIPIP,
+				},
+				Hostname: f.Hostname,
+			})
+			Expect(err).NotTo(HaveOccurred(), "failed to assign IPIP tunnel address")
+		}
 	}
 }
 
 func setupWorkloads(infra infrastructure.DatastoreInfra, tc infrastructure.TopologyContainers, to infrastructure.TopologyOptions, client client.Interface, enableIPv6 bool) (w, w6, hostW, hostW6 [3]*workload.Workload) {
+	return setupWorkloadsWithOffset(infra, tc, to, client, enableIPv6, 0)
+}
+
+func setupWorkloadsWithOffset(infra infrastructure.DatastoreInfra, tc infrastructure.TopologyContainers, to infrastructure.TopologyOptions, client client.Interface, enableIPv6 bool, offset int) (w, w6, hostW, hostW6 [3]*workload.Workload) {
 	// Install a default profile that allows all ingress and egress, in the absence of any Policy.
 	infra.AddDefaultAllow()
 
-	// Wait until the vxlan device appears.
-	Eventually(func() error {
-		for i, f := range tc.Felixes {
-			out, err := f.ExecOutput("ip", "link")
-			if err != nil {
-				return err
-			}
-			if strings.Contains(out, "vxlan.calico") {
-				continue
-			}
-			return fmt.Errorf("felix %d has no vxlan device", i)
-		}
-		return nil
-	}, "10s", "100ms").ShouldNot(HaveOccurred())
+	if to.VXLANMode != api.VXLANModeNever {
+		waitForVXLANDevice(tc, enableIPv6)
+	}
 
-	if enableIPv6 {
-		Eventually(func() error {
-			for i, f := range tc.Felixes {
-				out, err := f.ExecOutput("ip", "link")
-				if err != nil {
-					return err
-				}
-				if strings.Contains(out, "vxlan-v6.calico") {
-					continue
-				}
-				return fmt.Errorf("felix %d has no IPv6 vxlan device", i)
-			}
-			return nil
-		}, "10s", "100ms").ShouldNot(HaveOccurred())
+	if to.IPIPMode != api.IPIPModeNever {
+		waitForIPIPDevice()
 	}
 
 	// Create workloads, using that profile.  One on each "host".
@@ -1221,7 +1229,7 @@ func setupWorkloads(infra infrastructure.DatastoreInfra, tc infrastructure.Topol
 	_, IPv6CIDR, err := net.ParseCIDR(to.IPv6PoolCIDR)
 	Expect(err).To(BeNil())
 	for ii := range w {
-		wIP := fmt.Sprintf("%d.%d.%d.2", IPv4CIDR.IP[0], IPv4CIDR.IP[1], ii)
+		wIP := fmt.Sprintf("%d.%d.%d.2", IPv4CIDR.IP[0], IPv4CIDR.IP[1], ii+offset)
 		wName := fmt.Sprintf("w%d", ii)
 		err := client.IPAM().AssignIP(context.Background(), ipam.AssignIPArgs{
 			IP:       net.MustParseIP(wIP),
@@ -1237,7 +1245,7 @@ func setupWorkloads(infra infrastructure.DatastoreInfra, tc infrastructure.Topol
 		w[ii].ConfigureInInfra(infra)
 
 		if enableIPv6 {
-			w6IP := fmt.Sprintf("%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%d:3", IPv6CIDR.IP[0], IPv6CIDR.IP[1], IPv6CIDR.IP[2], IPv6CIDR.IP[3], IPv6CIDR.IP[4], IPv6CIDR.IP[5], IPv6CIDR.IP[6], IPv6CIDR.IP[7], IPv6CIDR.IP[8], IPv6CIDR.IP[9], IPv6CIDR.IP[10], IPv6CIDR.IP[11], ii)
+			w6IP := fmt.Sprintf("%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%d:3", IPv6CIDR.IP[0], IPv6CIDR.IP[1], IPv6CIDR.IP[2], IPv6CIDR.IP[3], IPv6CIDR.IP[4], IPv6CIDR.IP[5], IPv6CIDR.IP[6], IPv6CIDR.IP[7], IPv6CIDR.IP[8], IPv6CIDR.IP[9], IPv6CIDR.IP[10], IPv6CIDR.IP[11], ii+offset)
 			w6Name := fmt.Sprintf("w6-%d", ii)
 			err := client.IPAM().AssignIP(context.Background(), ipam.AssignIPArgs{
 				IP:       net.MustParseIP(w6IP),
@@ -1265,3 +1273,36 @@ func setupWorkloads(infra infrastructure.DatastoreInfra, tc infrastructure.Topol
 
 	return
 }
+
+func waitForVXLANDevice(tc infrastructure.TopologyContainers, enableIPv6 bool) {
+	// Wait until the vxlan device appears.
+	Eventually(func() error {
+		for i, f := range tc.Felixes {
+			out, err := f.ExecOutput("ip", "link")
+			if err != nil {
+				return err
+			}
+			if strings.Contains(out, "vxlan.calico") {
+				continue
+			}
+			return fmt.Errorf("felix %d has no vxlan device", i)
+		}
+		return nil
+	}, "10s", "100ms").ShouldNot(HaveOccurred())
+
+	if enableIPv6 && !BPFMode() {
+		Eventually(func() error {
+			for i, f := range tc.Felixes {
+				out, err := f.ExecOutput("ip", "link")
+				if err != nil {
+					return err
+				}
+				if strings.Contains(out, "vxlan-v6.calico") {
+					continue
+				}
+				return fmt.Errorf("felix %d has no IPv6 vxlan device", i)
+			}
+			return nil
+		}, "10s", "100ms").ShouldNot(HaveOccurred())
+	}
+}
