@@ -15,6 +15,7 @@
 package clientv3
 
 import (
+	"bytes"
 	"context"
 	"fmt"
 	"net"
@@ -296,6 +297,47 @@ func (r ipPools) Watch(ctx context.Context, opts options.ListOptions) (watch.Int
 	return r.client.resources.Watch(ctx, opts, apiv3.KindIPPool, nil)
 }
 
+// cidrChangeOK returns true if the new CIDR is an acceptable update to the old one.
+//
+// Allowed cases:
+//   - The CIDRs match exactly.
+//   - The CIDRs are semantically identical and the new CIDR is simply the canonical
+//     form of the old one (i.e., the update only normalizes formatting).
+//
+// This accounts for clients that may have stored non-canonical CIDRs by bypassing
+// our validation logic, allowing updates that preserve the same network while
+// converting the CIDR to its canonical form.
+func cidrChangeOK(oldCIDR, newCIDR string) bool {
+	// 1. If strings match exactly, allow.
+	if oldCIDR == newCIDR {
+		return true
+	}
+
+	// Parse both
+	_, oldNet, errOld := net.ParseCIDR(oldCIDR)
+	_, newNet, errNew := net.ParseCIDR(newCIDR)
+
+	// If either fails to parse → reject
+	if errOld != nil || errNew != nil {
+		return false
+	}
+
+	// 2. Check semantic equality (same network IP + mask)
+	sameNetwork := oldNet.IP.Equal(newNet.IP) &&
+		bytes.Equal(oldNet.Mask, newNet.Mask)
+	if !sameNetwork {
+		return false
+	}
+
+	// 3. New CIDR must already be canonical (IPNet.String()); don't allow introducing
+	// a non-canonical CIDR.
+	if newCIDR != newNet.String() {
+		return false
+	}
+
+	return true
+}
+
 // validateAndSetDefaults validates IPPool fields and sets default values that are
 // not assigned.
 // The old pool will be unassigned for a Create.
@@ -333,9 +375,9 @@ func (r ipPools) validateAndSetDefaults(ctx context.Context, new, old *apiv3.IPP
 	}
 
 	// If there was a previous pool then this must be an Update, validate that the
-	// CIDR has not changed.  Since we are using normalized CIDRs we can just do a
-	// simple string comparison.
-	if old != nil && old.Spec.CIDR != new.Spec.CIDR {
+	// CIDR has not changed. Use semantic comparison to handle different textual
+	// representations of the same CIDR (e.g., IPv6 with different zero compression).
+	if old != nil && !cidrChangeOK(old.Spec.CIDR, new.Spec.CIDR) {
 		errFields = append(errFields, cerrors.ErroredField{
 			Name:   "IPPool.Spec.CIDR",
 			Reason: "IPPool CIDR cannot be modified",
